@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $superduper: decoder.c,v 1.9 2004/05/16 17:08:05 sam Exp $
+ * $superduper: decoder.c,v 1.10 2004/05/26 11:35:12 sam Exp $
  */
 #include <assert.h>
 #include <pthread.h>
@@ -26,6 +26,7 @@
 #include "input.h"
 
 #define BUFFER_SIZE 16*1024
+#define N_AVERAGE_FRAMES 10
 /*#define DEBUG*/
 
 /**
@@ -97,7 +98,9 @@ scan_file (struct mad_info_t *info, gboolean fast)
   struct mad_stream stream;
   struct mad_header header;
   int remainder = 0;
+  int data_used = 0;
   int len = 0;
+  int tagsize = 0;
   unsigned char buffer[BUFFER_SIZE];
   struct mad_frame frame; /* to read xing data */
   gboolean has_xing = FALSE;
@@ -120,6 +123,7 @@ scan_file (struct mad_info_t *info, gboolean fast)
       memcpy (buffer, stream.this_frame, remainder);
       len = input_get_data (info, buffer + remainder,
                             BUFFER_SIZE - remainder);
+
       if (len <= 0)
         break;
 
@@ -146,7 +150,7 @@ scan_file (struct mad_info_t *info, gboolean fast)
 	      if (stream.error == MAD_ERROR_LOSTSYNC)
 		{
 		  /* ignore LOSTSYNC due to ID3 tags */
-		  int tagsize = id3_tag_query (stream.this_frame,
+		  tagsize = id3_tag_query (stream.this_frame,
 		      stream.bufend -
 		      stream.this_frame);
 		  if (tagsize > 0)
@@ -165,7 +169,8 @@ scan_file (struct mad_info_t *info, gboolean fast)
 	    }
 	  info->frames++;
           mad_timer_add (&info->duration, header.duration);
-          if (info->frames == 1) 
+          data_used += stream.next_frame - stream.this_frame;
+          if (info->frames == 1)
 	    {
 	      /* most of these *should* remain constant */
 	      info->bitrate = header.bitrate;
@@ -182,16 +187,16 @@ scan_file (struct mad_info_t *info, gboolean fast)
 	          if (xing_parse (&info->xing, stream.anc_ptr, stream.anc_bitlen) == 0) 
 	            {
 #ifdef DEBUG
-	              g_message ("found xing header");
+		      g_message ("found xing header");
 #endif /* DEBUG */
-                      has_xing = TRUE;
-                      info->vbr = TRUE; /* otherwise xing header would have been 'Info' */
+		      has_xing = TRUE;
+		      info->vbr = TRUE; /* otherwise xing header would have been 'Info' */
 		      info->frames = info->xing.frames;
 		      mad_timer_multiply (&info->duration, info->frames);
-                      info->bitrate = 8.0 * info->xing.bytes / mad_timer_count(info->duration, MAD_UNITS_SECONDS); 
-                      break;
-                    }
-	        }
+		      info->bitrate = 8.0 * info->xing.bytes / mad_timer_count(info->duration, MAD_UNITS_SECONDS); 
+		      break;
+		    }
+		}
 
 	    }
 	  else 
@@ -210,14 +215,21 @@ scan_file (struct mad_info_t *info, gboolean fast)
 		g_warning ("number of channels varies!!");
 	    }
 
-	  if (fast && info->frames > 10)
+	  if (fast && info->frames >= N_AVERAGE_FRAMES)
 	    {
+              float frame_size = ((double)data_used) / N_AVERAGE_FRAMES;
+	      info->frames = (info->size - tagsize) / frame_size;
+              //int frame_frac = (info->size - tagsize) % frame_size;
 #ifdef DEBUG
-	       g_message ("using fast playtime calculation");
+	      g_message ("using fast playtime calculation");
+              g_message ("data used = %d [tagsize=%d]", data_used, tagsize);
+              //g_message ("frame_size = %f [frac=%d]", frame_size, frame_frac);
+              g_message ("total frames = %d", info->frames);
 #endif /* DEBUG */
-	       info->frames = info->size / (stream.next_frame - stream.this_frame);
-	       mad_timer_multiply (&info->duration, info->frames);
-	       break;
+              info->duration.seconds /= N_AVERAGE_FRAMES;
+              info->duration.fraction /= N_AVERAGE_FRAMES;
+	      mad_timer_multiply (&info->duration, info->frames);
+	      break;
 	    }
 	}
       if (stream.error != MAD_ERROR_BUFLEN)
